/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Birdstep LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file pspplatdetect.h
 *  \brief Header for the Platform Support Package - Compiler dependent detection of platform
 */

/* clang-format off */

#ifndef RDM_DOXYGEN
#if !defined(PSPPLATDETECT_H_INCLUDED_)
    #define PSPPLATDETECT_H_INCLUDED_
    #if defined(SIZEOF_VOIDP) && (SIZEOF_VOIDP == 8)
        #define RDM_64BIT
    #endif

    /* The following is compiled from:
       http://sourceforge.net/p/predef/wiki/Home/ */
    #if defined(RDM_NO_OS)
      #undef RDM_NO_OS
      #if defined(RDM_64BIT)
        #define RDM_NO_OS_64
      #else
        #define RDM_NO_OS_32
      #endif
    #elif defined(RDM_FREE_RTOS)
      #undef RDM_FREE_RTOS
      #if defined(RDM_64BIT)
        #define RDM_FREE_RTOS_64
      #else
        #define RDM_FREE_RTOS_32
      #endif
    #elif defined(__ANDROID__)
        #if defined(__arm__) /* GCC */
            #define RDM_ANDROID_ARM
        #else
            #error No architecture defined for Android
        #endif
    #elif defined(__linux__)
        #if defined(__arm__) /* GCC */
            #define RDM_LINUX_ARM
        #elif defined(__aarch64__) /* GCC */
            #define RDM_LINUX_AARCH64
        #elif defined(i386) || defined(__i386__) /* GCC */ || \
            defined(__x86_64) || defined(__x86_64__) /* Intel C/C++ */
            #if defined(RDM_64BIT)
                #define RDM_LINUX_X64
            #else
                #define RDM_LINUX_X86
            #endif
        #elif defined(__ia64__) || defined(_IA64) || defined(__IA64__) /* GCC */
            #define RDM_LINUX_ITANIUM_64
        #elif defined (__powerpc) || defined (__powerpc__) || defined (__POWERPC__) || defined (__ppc__) /* GCC */
            #define RDM_LINUX_POWERPC
       #else
            #error No architecture defined for Linux
        #endif
    #elif defined(__QNX__) /* QNX 4.x */ || \
        defined(__QNXNTO__) /* QNX 6.x */
        #if defined(__arm__) /* GCC */
            #define RDM_NEUTRINO_ARM
        #elif defined(__aarch64__) /* GCC */
            #define RDM_NEUTRINO_ARM64
        #elif defined(i386) || defined(__i386__) /* GCC */
            #define RDM_NEUTRINO_X86
        #elif defined (__x86_64__)
            #define RDM_NEUTRINO_X86_64
        #elif defined (__PPC__) /* GCC */
            #define RDM_NEUTRINO_PPC
        #else
            #error No architecture defined for QNX Neutrino
        #endif
    #elif defined(__APPLE__) & defined(__MACH__)
        #if defined(__arm__) /* GCC */
            #define RDM_IPHONE_ARM
        #elif defined(i386) || defined(__i386__) /* GCC */
            #if defined(RDM_64BIT)
                #define RDM_OSX_X64
            #else
                #define RDM_OSX_X86
            #endif
        #elif defined(__x86_64) || defined(__x86_64__ ) /* GCC */
                #define RDM_OSX_X64
        #else
            #error No architecture defined for Apple OSX/IOS
        #endif
    #elif defined(sun) || defined(__sun)
        #if defined(__SVR4) || defined(__svr4__) /* Solaris */
            #if defined(__sparc) /* Sun Studio */ || \
                defined(__x86_64) || defined(__x86_64__) /* Intel C/C++ */ || \
                defined(__sparc__) /* GCC */
                #if defined(RDM_64BIT)
                    #define RDM_SOLARIS_SPARC_64
                #else
                    #define RDM_SOLARIS_SPARC_32
                #endif
            #elif defined(__i386) /* Sun Studio */ || \
                defined(__i386__) || defined(i386)
                #if defined(RDM_64BIT)
                    #define RDM_SOLARIS_X64
                #else
                    #define RDM_SOLARIS_X86
                #endif
            #else
                #error No architecture defined for Solaris
            #endif
        #else
            #error SunOS not supported
        #endif
    #elif defined(_WIN32)
        #if defined (_WIN64)
            #define RDM_WINDOWS_X64
        #else
            #define RDM_WINDOWS_X86
        #endif
    #elif defined(__FreeBSD__)
        #if defined(RDM_64BIT)
            #define RDM_FREEBSD_X64
        #else
            #define RDM_FREEBSD_X86
        #endif
    #elif defined(__INTEGRITY)
        #if defined(__i386) || defined(__i386__)
            #define RDM_INTEGRITY_X86
        #elif defined (__x86_64__)
            #define RDM_INTEGRITY_X64
        #elif defined (__ARM) || defined (__ARM__)
            #define RDM_INTEGRITY_ARM
        #elif defined (__ppc__) || defined (__ppc) || defined (__PPC__)
            #define RDM_INTEGRITY_POWERPC
        #else
            #error No architecture defined for INTEGRITY
        #endif
    #elif defined(__VXWORKS__)
        #if defined(_WRS_KERNEL)
            #if defined (__DCC__) /* Diab */
                #error RDM have not been ported to VxWorks Kernel modules using the Diab Compiler
            #elif defined(i386) || defined(__i386__) /* GCC */
                #define RDM_VXWORKS_X86
            #elif defined (__powerpc) || defined (__powerpc__) || defined (__POWERPC__) || defined (__ppc__) || defined (__ppc) /* GCC */
                #define RDM_VXWORKS_POWERPC
            #elif defined(__sh__) /* GCC */
                #if defined(__SH4__)
                    #define RDM_VXWORKS_SH4 /* We should really redefine this to SH4 or SH */
                #endif
            #elif defined(__arm__) /* GCC */
                #define RDM_VXWORKS_ARM
            #else
                #error No architecture defined for VxWorks Kernel Modules using the GNU compiler
            #endif
        #elif defined (__RTP__)
            #if defined (__DCC__) /* Diab */
                #if defined (__ppc) /* Diab */
                    #define RDM_VXWORKS_RTP_POWERPC
                #elif defined(__386) /* Diab */
                    #define RDM_VXWORKS_RTP_X86
                #else
                    #error No architecture defined for VxWorks RTP using the Diab compiler
                #endif
            #elif defined(i386) || defined(__i386__) /* GCC */
                #define RDM_VXWORKS_RTP_X86
            #elif defined (__powerpc) || defined (__powerpc__) || defined (__POWERPC__) || defined (__ppc__) || defined (__ppc) /* GCC */
                #define RDM_VXWORKS_RTP_POWERPC
            #elif defined(__sh__) /* GCC */
                #if defined(__SH4__)
                    #define RDM_VXWORKS_RTP_SH4 /* We should really redefine this to SH4 or SH */
                #endif
            #elif defined(__arm__) /* GCC */
                #define RDM_VXWORKS_RTP_ARM
            #else
                #error No architecture defined for VxWorks RTP using the GNU compiler
            #endif
        #endif
    #else
        #error No architecture defined for unknown OS
    #endif
    #undef RDM_64BIT
#endif /* PSPPLATDETECT_H_INCLUDED_ */
#endif
